<?php
/*
Plugin Name: BuddyPress Group Twitter
Plugin URI: http://wordpress.org/extend/plugins/buddypress-group-twitter
Description: Link Twitter accounts to groups and track new tweets.
Version: 1.0.2
Requires at least: WPMU 2.8.5 / BuddyPress 1.1.2
Tested up to: WPMU 2.8.6 / BuddyPress 1.1.2
License: GNU/GPL 2
Author: Andy Peatling
Author URI: http://buddypress.org/developers/apeatling
Site Wide Only: true
*/

class BP_Group_Twitter extends BP_Group_Extension {

	function bp_group_twitter() {
		global $bp;

		$this->name = __( 'Group Twitter Accounts', 'bp-group-twitter' );
		$this->slug = 'twitter';
		$this->nav_item_name = __( 'Twitter', 'bp-group-twitter' );

		$this->create_step_position = 21;
		$this->nav_item_position = 31;

		$this->enable_nav_item = $this->has_twitter_accounts();
	}

	function create_screen() {
		global $bp;

		if ( !bp_is_group_creation_step( $this->slug ) )
			return false;
		?>

		<p><?php _e(
			"Add Twitter account names that you'd like to attach to this group in the box below.
			 Any future tweets by these users will show up on the group page.", 'bp-group-twitter' ) ?>
		</p>

		<p class="desc"><?php _e( "Separate accounts with commas. Example: @apeatling, @buddypressdev, @wordpress", 'bp-group-twitter' ) ?></p>

		<p>
			<label for="twitteraccounts"><?php _e( "Twitter Accounts:", 'bp-group-twitter' ) ?></label>
			<textarea name="twitteraccounts" id="twitteraccounts"><?php echo attribute_escape( implode( ', ', (array)groups_get_groupmeta( $bp->groups->current_group->id, 'twitteraccounts' ) ) ) ?></textarea>
		</p>

		<?php
		wp_nonce_field( 'groups_create_save_' . $this->slug );
	}

	function create_screen_save() {
		global $bp;

		check_admin_referer( 'groups_create_save_' . $this->slug );

		$unfiltered_accounts = explode( ',', $_POST['twitteraccounts'] );

		foreach( (array) $unfiltered_accounts as $twitter_account ) {
			if ( !empty( $twitter_account ) )
				$twitter_accounts[] = trim( $twitter_account );
		}

		groups_update_groupmeta( $bp->groups->current_group->id, 'twitteraccounts', $twitter_accounts );
	}

	function edit_screen() {
		global $bp;

		if ( !bp_is_group_admin_screen( $this->slug ) )
			return false; ?>

		<h2><?php echo attribute_escape( $this->name ) ?></h2>

		<p class="desc">
			<?php _e( "Add Twitter account names that you'd like to attach to this group in the box below.
			 		   Any future tweets by these users will show up on the group page.", 'bp-group-twitter' ) ?>
		</p>

		<p>
			<label for="twitteraccounts"><?php _e( "Twitter Accounts:", 'bp-group-twitter' ) ?></label>
			<textarea name="twitteraccounts" id="twitteraccounts"><?php echo attribute_escape( implode( ', ', (array)groups_get_groupmeta( $bp->groups->current_group->id, 'twitteraccounts' ) ) ) ?></textarea>
		</p>

		<input type="submit" name="save" value="<?php _e( "Update Accounts", 'bp-group-twitter' ) ?>" />

		<?php
		wp_nonce_field( 'groups_edit_save_' . $this->slug );
	}

	function edit_screen_save() {
		global $bp;

		if ( !isset( $_POST['save'] ) )
			return false;

		check_admin_referer( 'groups_edit_save_' . $this->slug );

		$unfiltered_accounts = explode( ',', $_POST['twitteraccounts'] );

		foreach( (array) $unfiltered_accounts as $twitter_account ) {
			if ( !empty( $twitter_account ) )
				$twitter_accounts[] = trim( $twitter_account );
		}

		/* Get a list of the twitter accounts before they were updated */
		$orig_twitter_accounts = groups_get_groupmeta( $bp->groups->current_group->id, 'twitteraccounts' );

		/* Loop and find any accounts that have been removed, so we can delete activity stream items */
		foreach( (array) $orig_twitter_accounts as $account ) {
			if ( !in_array( $account, (array) $twitter_accounts ) )
				$removed[] = $account;
		}

		/* Remove tweets for these accounts from the activity stream */
		if ( $removed ) {
			include_once( ABSPATH . WPINC . '/rss.php' );

			/* Remove activity for accounts that have been removed. */
			foreach( (array) $removed as $username ) {
				if ( $tweets = fetch_rss( "http://twitter.com/statuses/user_timeline/$username.rss" ) ) {
					/* Check there are tweets */
					if ( empty( $tweets->items ) )
						return false;

					foreach ( $tweets->items as $tweet ) {
						if ( function_exists( 'bp_activity_delete_by_item_id' ) ) {
							bp_activity_delete_by_item_id( array(
								'item_id' => $bp->groups->current_group->id,
								'secondary_item_id' => wp_hash( $username . ':' . wp_filter_kses( $tweet['link'] ) ),
								'component_name' => $bp->groups->id,
								'component_action' => 'new_tweet'
							) );
						}
					}
				}
			}
		}

		groups_update_groupmeta( $bp->groups->current_group->id, 'twitteraccounts', $twitter_accounts );
		groups_update_groupmeta( $bp->groups->current_group->id, 'last_activity', time() );

		/* Re-schedule the feed update event */
//		wp_schedule_event( strtotime( '+10 seconds' ), 'twicedaily', create_function( '', 'bp_groupblogs_fetch_group_feeds(' . $bp->groups->current_group->id . ');' ) );

		bp_core_add_message( __( 'Twitter accounts updated successfully!', 'bp-group-twitter' ) );

		bp_core_redirect( bp_get_group_permalink( $bp->groups->current_group ) . '/admin/' . $this->slug );
	}

	function display() { ?>
		<div id="group-twitter-placeholder">
			<span class="ajax-loader" style="display: block; padding-left: 25px"> <?php _e( 'Fetching Tweets', 'bp-group-twitter' ) ?></span>
		</div>
	<?php
	}

	function has_twitter_accounts() {
		global $bp;

		$twitteraccounts = groups_get_groupmeta( $bp->groups->current_group->id, 'twitteraccounts' );

		if ( !empty( $twitteraccounts ) )
			return true;

		return false;
	}

	function widget_display() {}
}
bp_register_group_extension( 'BP_Group_Twitter' );


/* Fetch and record in activity stream */
function bp_group_twitter_fetch( $usernames, $object, $item_id ) {
	global $bp;

	include_once(ABSPATH . WPINC . '/rss.php');

	/* Fetch the tweets for each username */
	foreach( (array) $usernames as $username ) {
		if ( !$tweets = fetch_rss( "http://twitter.com/statuses/user_timeline/$username.rss" ) )
			return false;

		/* Check there are tweets */
		if ( empty( $tweets->items ) )
			return false;

		/* Oldest first please. */
		$tweets->items = array_reverse( $tweets->items );

		foreach ( $tweets->items as $tweet ) {
			$content = utf8_encode( substr( strstr( $tweet['description'], ': ' ), 2, strlen( $tweet['description'] ) ) );
			$link = wp_filter_kses( $tweet['link'] );
			$pubdate = $tweet['pubdate'];

			/* Auto-hyperlink URL's and Twitter users */
			$content = make_clickable( $content );
			$content = twitterize( $content );

			/* Filter the content for nasties */
			$content = wp_filter_kses( $content );

			/* Add a permalink link # at the end of each tweet */
			$content = "$content <a href='$link'>#</a>";

			/* Now post it to the activity stream */
			$activity_content = sprintf( __( 'New twitter post by <a href="%s">%s</a> in the group <a href="%s">%s</a>:', 'bp-group-twitter' ), "http://twitter.com/" . substr( $username, 1, strlen( $username ) ), $username, bp_get_group_permalink( $bp->groups->current_group ), attribute_escape( $bp->groups->current_group->name ) );
			$activity_content .= '<blockquote>' . $content . '</blockquote>';
			$activity_content = apply_filters( 'bp_group_twitter_activity_content', $activity_content, &$post, &$group );

			/* Record in activity streams if this hasn't been already. */
			if ( function_exists( 'bp_activity_check_exists_by_content' ) && !bp_activity_check_exists_by_content( $activity_content ) ) {
				groups_record_activity( array(
					'user_id' => false,
					'content' => $activity_content,
					'primary_link' => $link,
					'component_name' => $object,
					'component_action' => 'new_tweet',
					'item_id' => $item_id,
					'secondary_item_id' => wp_hash( $username . ':' . $link ),
					'recorded_time' => strtotime( $pubdate )
				) );
			}
		}
	}
}

function twitterize($text) {
	$text = preg_replace('/([\.|\,|\:|\¡|\¿|\>|\{|\(]?)@{1}(\w*)([\.|\,|\:|\!|\?|\>|\}|\)]?)\s/i', "$1<a href=\"http://twitter.com/$2\" class=\"twitter-user\">@$2</a>$3 ", $text);
	return $text;
}

function bp_group_twitter_display() {
	global $bp;

	$twitter_accounts = groups_get_groupmeta( $bp->groups->current_group->id, 'twitteraccounts' );

	if ( empty( $twitter_accounts ) )
		return false;

	/* Fetch the latest twitter posts */
	bp_group_twitter_fetch( $twitter_accounts, $bp->groups->id, $bp->groups->current_group->id );

	if ( bp_has_activities( 'object=groups&primary_id=' . $bp->groups->current_group->id . '&max=150&per_page=25&action=new_tweet' ) ) : ?>

		<div class="bp-widget">

			<div class="group-twitter-pagination">
				<div class="pag-count" id="activity-count">
					<?php bp_activity_pagination_count() ?>
				</div>

				<div class="pagination-links" id="activity-pag">
					&nbsp; <?php bp_activity_pagination_links() ?>
				</div>
			</div>

			<ul id="activity-list" class="activity-list item-list">
			<?php while ( bp_activities() ) : bp_the_activity(); ?>
				<li class="<?php bp_activity_css_class() ?>">
					<div class="activity-avatar">
						<?php bp_activity_avatar() ?>
					</div>

					<?php bp_activity_content() ?>
				</li>
			<?php endwhile; ?>
			</ul>

		</div>

	<?php endif;
}
add_action( 'wp_ajax_load_group_twitter', 'bp_group_twitter_display' );

function bp_group_twitter_load_cssjs() {
	global $bp;

	if ( !$bp->groups->current_group )
		return false;
?>
	<script type="text/javascript">
		jQuery(document).ready( function() {
			jQuery('#group-twitter-placeholder').ready( function() {
				jQuery.post( ajaxurl, {
					action: 'load_group_twitter',
					'cookie': encodeURIComponent(document.cookie)
				},
				function(response)
				{
					jQuery("div#group-twitter-placeholder").html(response).hide().fadeIn(150);
				});
			});

			jQuery('.group-twitter-pagination a').livequery( 'click', function() {
				jQuery('.ajax-loader').toggle();

				var ebpage = jQuery(this).attr('href');
				ebpage = ebpage.split('=');

				jQuery.post( ajaxurl, {
					action: 'load_group_twitter',
					'cookie': encodeURIComponent(document.cookie),
					'acpage': ebpage[1]
				},
				function(response)
				{
					jQuery('.ajax-loader').toggle();

					response = response.substr(0, response.length-1);

					jQuery("div#group-twitter-placeholder").fadeOut(150,
						function() {
							jQuery("div#group-twitter-placeholder").html(response);
							jQuery("div#group-twitter-placeholder").fadeIn(150);
						}
					);

					return false;
				});

			return false;

			});
		});
	</script>

	<style type="text/css">
		li a#nav-twitter {
			background: url(<?php echo content_url( 'plugins/buddypress-group-twitter/icon.gif' ) ?>) 87% 52% no-repeat;
			padding: 0.55em 3em 0.55em 0 !important;
			margin-right: 0.85em !important;
		}
	</style>
	<?php
}
add_action( 'wp_footer', 'bp_group_twitter_load_cssjs' );

?>